from functional_notations import _F


@_F
def csv(csv_str, **kwargs):
    """
    :param csv_str:
    :param kwargs:
    :return: DataFrame object
    """
    data = None
    # under construction
    return data


@_F
def yaml(yaml_str, _ruamel, **kwargs):
    """

    :param yaml_str:
    :param _ruamel:
    :param kwargs:
    :return: DataFrame object
    """
    data = None
    return data


@_F
def table(yaml_str, _ruamel, **kwargs):
    """

    :param yaml_str:
    :param _ruamel:
    :param kwargs:
    :return: DataFrame object
    """
    data = None
    return data
